/****************************************************************************
Annotation.h
    is part of TeclMapVis

    Copyright 2014 Active Vision Group
              Universitaet Koblenz-Landau

    Author:  Marcel Haeselich
    Contact: Marcel Haeselich, mhaeselich[at]uni-koblenz.de


    TeclMapVis is free software: you can redistribute it and/or modify
    it under the terms of thr GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.
    TeclMapVis is distributed in the hope that it will be usefull,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public Livense for more details.
    You should have received a copy of the GNU General Public License
    along with TeclMapVis. If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

#ifndef ANNOTATION_H
#define ANNOTATION_H

#include <Qt/QtOpenGL>
#include <memory>

#include "PointCloud.h"
#include "Camera.h"


class Annotation
{
public:
    Annotation();


    void initialize(std::vector<std::shared_ptr<PMap::PointCloud3d > >* pointClouds,
                    std::vector<bool>*                                  activePointClouds,
                    std::function<void(int)>                            activatePointCloud);

    //! set Camera-Pointer, to access current Camera Position
    void setCamera(Camera* camera){
        m_camera = camera;}

    //! changes current label
    void setLabel(std::tuple<float,float, float> label){
        m_currentLabel = label;}

    //! renders the pen
    void render_pen(QPoint cursor_pos);

    //! changes point-cloud-labels
    void draw(QPointF mouseXY);

    //! checks if a origion is double-clicked to de/activate this Pointcloud
    void checkForPCActivation(QPointF mouseXY);

    //! key Events
    void keyPressEvent(QKeyEvent* e);
    void keyReleaseEvent(QKeyEvent* e);

    //! mouse Events
    void mousePressEvent(QMouseEvent* e);
    void mouseMoveEvent(QMouseEvent* e);
    void mouseReleaseEvent(QMouseEvent* e);
    void wheelEvent(QWheelEvent* e);
    void mouseDoubleClickEvent(QMouseEvent* e);

private:

    // control-members
    bool m_ctrlDown;
    bool m_shiftDown;
    bool m_mouseDown;

    // pen-members
    float m_pen_angle;          // ~ pen radius
    float m_pen_range;
    float m_max_penrange;
    int   m_show_penrange;      // number of pictures pen-range-info-text is shown

    Camera* m_camera;

    std::vector<std::shared_ptr<PMap::PointCloud3d > >* m_PointClouds;
    std::vector<bool>*                                  m_activePointClouds;
    std::tuple<float,float, float>                      m_currentLabel;

    std::function<void(int)>                            f_activatePointCloud;

};

#endif



