#include "CVUtil.h"
#include <opencv2/highgui/highgui.hpp>

namespace
{
//-----------------------------------------------------------------------------
void my_mouse_callback( int event, int x, int y, int flags, void* user )
{
	(*(std::function<void(int,int,int,int)>*)user)(event,x,y,flags);
}
//-----------------------------------------------------------------------------
}

namespace PMap
{
//-----------------------------------------------------------------------------
std::vector<Eigen::Matrix<double,2,1,Eigen::DontAlign> > selectPointsFromImage(const cv::Mat& img)
{
	const char* windowName="Select Points";
	const int zoomFactor=10;
	const int zoomSize=100;
    cv::namedWindow(windowName,cv::WINDOW_NORMAL);
	std::vector<Eigen::Matrix<double,2,1,Eigen::DontAlign> > ret;

	bool zoomed=false;
	cv::Rect zoomRoi;

	std::function<void(int, int, int, int)> fn=[&](int event, int x, int y, int flags)
	{
		if (event==CV_EVENT_LBUTTONUP)
		{
			if (zoomed)
			{
				if ((x<0)||(x/zoomFactor>zoomRoi.width)||(y<0)||(y/zoomFactor>zoomRoi.height))
					return;

				double gx=x/static_cast<double>(zoomFactor)+zoomRoi.x;
				double gy=y/static_cast<double>(zoomFactor)+zoomRoi.y;
				ret.push_back({gx,gy});

				cv::imshow(windowName,img);
				zoomed=false;
			}
			else
			{
				if ((x<0)||(x>img.cols)||(y<0)||(y>img.rows))
					return;

				const int minx=std::max(0,x-zoomSize);
				const int miny=std::max(0,y-zoomSize);
				const int maxx=std::min(img.cols,x+zoomSize);
				const int maxy=std::min(img.rows,y+zoomSize);
				zoomRoi=cv::Rect(minx,miny,maxx-minx,maxy-miny);
				cv::Mat roi(img,zoomRoi);

				cv::Mat roiBig;
				cv::resize(roi,roiBig,cv::Size(roi.cols*zoomFactor,roi.rows*zoomFactor));
				cv::imshow(windowName,roiBig);
				zoomed=true;
			}
		}
		else if (event==CV_EVENT_RBUTTONUP)
		{
			cv::imshow(windowName,img);
			zoomed=false;
		}
	};
	cv::setMouseCallback(windowName,my_mouse_callback,&fn);

	cv::imshow(windowName,img);

	while (1)
	{
		if( cv::waitKey( 15 )%256==27 )
			break;
	}
	cv::destroyWindow(windowName);

	return ret;
}
//-----------------------------------------------------------------------------
}
