/****************************************************************************
Camera.h
    is part of TeclMapVis

    Copyright 2014 Active Vision Group
              Universitaet Koblenz-Landau

    Author:  Marcel Haeselich
    Contact: Marcel Haeselich, mhaeselich[at]uni-koblenz.de


    TeclMapVis is free software: you can redistribute it and/or modify
    it under the terms of thr GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.
    TeclMapVis is distributed in the hope that it will be usefull,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public Livense for more details.
    You should have received a copy of the GNU General Public License
    along with TeclMapVis. If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

#ifndef CAMERA_H
#define CAMERA_H

#include <Eigen/Core>
#include <Eigen/Geometry>

class Camera
{
public:
	Camera();
	~Camera();

	//! Strafes the camera sidewards by the specified amount
    void strafeCamera(double dist);
	//! Trucks the camera z-plane for-/backwards by specified amount parallel to z-plane
    void truckCamera(double dist);
	//! Moves the camera for-/backwards by the specified amount
    void moveCamera(double dist);

	//! Modifies the mouse sensitivity. Default: 1.0/170.0
    void setMouseSensitivity(double sens);

	//! Rotates the current view given a delta mouse movement
	void setViewByMouse(int dx, int dy);

	void setPos(const Eigen::Vector3d& pos);
	//! Set LookAt Point
	void setLookAt(const Eigen::Vector3d& lookAt);
	void setUp(const Eigen::Vector3d& up);

	const Eigen::Vector3d& getPos() const;
	//! Get LookAt Point
	const Eigen::Vector3d& getLookAt() const;
	const Eigen::Vector3d& getUp() const;

	Eigen::Quaterniond getQuat() const;

	//! Get mouse sensitivity
    double getSensitivity() const;
private:
	void rotateView(const Eigen::Vector3d& axis, double angle);
    double          m_sensitivity;

	Eigen::Vector3d m_pos;
	Eigen::Vector3d m_lookAt;
	Eigen::Vector3d m_up;
};

#endif

