/****************************************************************************
MainWindow.h
    is part of TeclMapVis

    Copyright 2014 Active Vision Group
              Universitaet Koblenz-Landau

    Author:  Marcel Haeselich
    Contact: Marcel Haeselich, mhaeselich[at]uni-koblenz.de


    TeclMapVis is free software: you can redistribute it and/or modify
    it under the terms of thr GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.
    TeclMapVis is distributed in the hope that it will be usefull,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public Livense for more details.
    You should have received a copy of the GNU General Public License
    along with TeclMapVis. If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/



#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <memory>
#include <boost/filesystem.hpp>
#include <QListWidget>

#include "PointCloudRenderer.h"

#include "Annotation.h"


// -- terrain - labels -----------------
static const std::tuple<float,float,float> UNKNOWN         = std::make_tuple(1.0f,1.0f,1.0f);
static const std::tuple<float,float,float> FALSE_UNKNOWN   = std::make_tuple(0.0f,0.0f,1.0f);
static const std::tuple<float,float,float> OBSTACLE        = std::make_tuple(1.0f,0.0f,0.0f);
static const std::tuple<float,float,float> FALSE_OBSTACLE  = std::make_tuple(0.8f,0.2f,0.2f);
static const std::tuple<float,float,float> STREET          = std::make_tuple(0.7f,0.7f,0.7f);
static const std::tuple<float,float,float> INFERRED_STREET = std::make_tuple(0.8f,0.8f,0.8f);
static const std::tuple<float,float,float> ROUGH           = std::make_tuple(0.4f,0.2f,0.1f);
static const std::tuple<float,float,float> INFERRED_ROUGH  = std::make_tuple(0.5f,0.3f,0.1f);
// -------------------

namespace Ui
{
	class MainWindow;
}

class MainWindow : public QMainWindow
{
	Q_OBJECT
	
public:
	explicit MainWindow(QWidget *parent = 0);
	~MainWindow();

public slots:
    //! whenever a new dataset is requested by the "Load dataset..."-button
	void loadDataset();

    //! whenever the selection or annotation changes
    void loadVelodata();

    //! whenever the "Save dataset"-button is pressed to write all annotations to the .label files
    void saveDataset();

    //! whenever the "Point size:" value changes
    void pointSizeChanged(int val);

    //! whenever the "label" changes
    void labelChanged(QString label);

    //! whenever the "show Points" checkbox changes
    void showPointsChanged(int state);

    //! whenever a key is pressed/released to delegate it to the Camera and Annotation
	void keyPressEvent(QKeyEvent* e);
    void keyReleaseEvent(QKeyEvent* e);

private:

    void activatePointCloud(int i);

    std::unique_ptr<Ui::MainWindow>                     m_ui;
    QString                                             m_MapFile = "";

    std::vector<std::shared_ptr<PMap::PointCloud3d > >  m_PointClouds;          //central storage for all pointclouds, memorizes color annotations
    std::vector<bool>                                   m_ActivePointClouds;

    std::vector<std::unique_ptr<PointCloudRenderer > >  m_cloudRenderer;
    bool                                                m_renderHeightColors;

    Annotation                                          m_Annotation;


};

#endif // MAINWINDOW_H
