/****************************************************************************
PointCloudRenderer.h
    is part of TeclMapVis

    Copyright 2014 Active Vision Group
              Universitaet Koblenz-Landau

    Author:  Marcel Haeselich
    Contact: Marcel Haeselich, mhaeselich[at]uni-koblenz.de


    TeclMapVis is free software: you can redistribute it and/or modify
    it under the terms of thr GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.
    TeclMapVis is distributed in the hope that it will be usefull,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public Livense for more details.
    You should have received a copy of the GNU General Public License
    along with TeclMapVis. If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

#ifndef POINTCLOUDRENDERER_H
#define POINTCLOUDRENDERER_H

#include <GL/glew.h>
#include <Qt/QtOpenGL>
#include "PointCloud.h"
//#include "RenderCache.h"
#include <iostream>
#include <GL/glut.h>

class PointCloudRenderer
{
public:
    //-----------------------------------------------------------------------------
    PointCloudRenderer(std::shared_ptr<PMap::PointCloud3d>& pc) : pc_(pc), visibilityState_(true), renderHeight_(0)
	{
    }
    //-----------------------------------------------------------------------------
    void setPointSize(int s)
    {
        pointSize_ = (float)s;
    }
    //-----------------------------------------------------------------------------
    void setVisibility(bool state)
    {
        visibilityState_ = state;
    }
    //-----------------------------------------------------------------------------
    void setRenderHeightColors(bool* b)
    {
        renderHeight_ = b;
    }
    //-----------------------------------------------------------------------------
	void render()
	{
        if(visibilityState_)
        {
            glPointSize(pointSize_);

            glEnableClientState(GL_VERTEX_ARRAY);
            glEnableClientState(GL_COLOR_ARRAY);

            if(*renderHeight_)
                glColorPointer(3, GL_FLOAT, 0, pc_->getHeightColorArray());
            else
                glColorPointer(3, GL_FLOAT, 0, pc_->getColorArray());
            glVertexPointer(3, GL_FLOAT, 0, pc_->getVertexArray());
            glDrawArrays(GL_POINTS, 0, pc_->getPoints().cols());

            glDisableClientState(GL_VERTEX_ARRAY);
            glDisableClientState(GL_COLOR_ARRAY);
        }
    }

private:
	std::shared_ptr<PMap::PointCloud3d> pc_;
    float pointSize_;
    bool visibilityState_;
    bool* renderHeight_;
};

#endif
