/****************************************************************************
RenderWidget.h
    is part of TeclMapVis

    Copyright 2014 Active Vision Group
              Universitaet Koblenz-Landau

    Author:  Marcel Haeselich
    Contact: Marcel Haeselich, mhaeselich[at]uni-koblenz.de


    TeclMapVis is free software: you can redistribute it and/or modify
    it under the terms of thr GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.
    TeclMapVis is distributed in the hope that it will be usefull,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public Livense for more details.
    You should have received a copy of the GNU General Public License
    along with TeclMapVis. If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

#ifndef RENDERWIDGET_H
#define RENDERWIDGET_H

#include <map>
#include <string>
#include <GL/glew.h>
#include <Qt/QtOpenGL>
#include <functional>
#include "Camera.h"

#include "Annotation.h"

class RenderWidget : public QGLWidget
{
	Q_OBJECT
public:
	RenderWidget(QWidget *parent = 0);

    //! set Annotation-Pointer to render his Pen and delegate Key Events
    void setAnnotation(Annotation* annotation){
        m_Annotation = annotation;
        m_Annotation->setCamera(& m_cam);
    }

    //! choose the renderables(pointClouds)
    void addRenderable(const std::string& name, std::function<void()> fn);
	void removeRenderable(const std::string& name);
	void enableRenderable(const std::string& name, bool enabled);
	void clearRenderables();

    //! Key Events
	virtual void keyPressEvent(QKeyEvent* e);
	virtual void keyReleaseEvent(QKeyEvent* e);

protected:

	virtual void initializeGL();

    //! render the coordinate-Axes, renderables(pointclouds) and the Annotation-Pen
	virtual void paintGL();
	virtual void resizeGL(int w, int h);

    //! Mouse Events
	virtual void mousePressEvent(QMouseEvent* e);
	virtual void mouseMoveEvent(QMouseEvent* e);
	virtual void mouseReleaseEvent(QMouseEvent* e);
	virtual void mouseDoubleClickEvent(QMouseEvent* e);
	virtual void wheelEvent(QWheelEvent* e);

private:

    //! handles Mouse and Keyboard updates
    void handleInputs();

    Annotation*     m_Annotation;

    Camera          m_cam;

	struct
	{
		int x;
		int y;
    }               resolution;

	struct InputState
	{
		bool forwardDown;
		bool backwardDown;
		bool leftDown;
		bool rightDown;

		bool spaceDown;
		bool ctrlDown;

		bool leftMouseDown;
		bool rightMouseDown;

		int lastx;
		int lasty;
	};
    InputState      inputState;

	struct Renderable
	{
        std::string             name;
        bool                    enabled;
        std::function<void()>   fn;
	};

    std::map<std::string,Renderable> m_renderables;
};

#endif
